@@ -48,7 +48,7 @@
 #include <gnu_gama/local/results/text/reduced_observations_to_ellipsoid.h>
 #include <gnu_gama/local/results/text/residuals_observations.h>
 #include <gnu_gama/local/results/text/error_ellipses.h>
-#include <gnu_gama/local/results/text/test_linearization.h>
+#include <gnu_gama/local/test_linearization_visitor.h>
 #include <gnu_gama/local/xmlerror.h>
 
 namespace {
@@ -83,6 +83,10 @@ int help()
     "--cov-band   covariance matrix of adjusted parameters in XML output\n"
     "             n  = -1  for full covariance matrix (implicit value)\n"
     "             n >=  0  covariances are computed only for bandwidth n\n"
+    "--iterations maximum number of iterations allowed in the linearized\n"
+    "             least squares algorithm (implicit value is 5)\n"
+//  "--updated-xml input data with free coordinates updated after adjustment\n"
+//  "             and rejected observations removed\n"
     "--version\n"
     "--help\n\n";
 
@@ -112,6 +116,10 @@ int help()
     "--cov-band   covariance matrix of adjusted parameters in XML output\n"
     "             n  = -1  for full covariance matrix (implicit value)\n"
     "             n >=  0  covariances are computed only for bandwidth n\n"
+    "--iterations maximum number of iterations allowed in the linearized\n"
+    "             least squares algorithm (implicit value is 5)\n"
+//  "--updated-xml input data with free coordinates updated after adjustment\n"
+//  "             and rejected observations removed\n"
     "--version\n"
     "--help\n\n";
 #endif
@@ -147,6 +155,8 @@ int main(int argc, char **argv)
     const char* argv_svgout = 0;
     const char* argv_obsout = 0;
     const char* argv_covband = 0;
+    const char* argv_iterations = 0;
+    const char* argv_updated_xml = 0;
 
 #ifdef GNU_GAMA_LOCAL_SQLITE_READER
     const char* argv_confname = 0;
@@ -190,6 +200,8 @@ int main(int argc, char **argv)
         else if (name == "svg"       ) argv_svgout = c;
         else if (name == "obs"       ) argv_obsout = c;
         else if (name == "cov-band"  ) argv_covband = c;
+        else if (name == "iterations") argv_iterations = c;
+        else if (name == "updated-xml") argv_updated_xml = c;
 #ifdef GNU_GAMA_LOCAL_SQLITE_READER
         else if (name == "sqlitedb")               argv_sqlitedb = c;
         else if (name == "configuration")          argv_confname = c;
@@ -376,7 +388,18 @@ int main(int argc, char **argv)
         char c;
         if (istr >> c) return help();
 
-        IS->set_xml_covband(band);
+        IS->set_adj_covband(band);
+      }
+
+    if (argv_iterations)
+      {
+        std::istringstream istr(argv_iterations);
+        int iter = IS->max_linearization_iterations();
+        if (!(istr >> iter) || iter < 0) return help();
+        char c;
+        if (istr >> c) return help();
+
+        IS->set_max_linearization_iterations(iter);
       }
 
     if (argv_latitude)
@@ -498,39 +521,25 @@ int main(int argc, char **argv)
 
         if (network_can_be_adjusted)
           {
-            int iteration = 0;
-            do
+            IS->clear_linearization_iterations();
+            while (IS->next_linearization_iterations() &&
+                   TestLinearization(IS))
               {
-                if(++iteration > 1)
-                  {
-                    // removed in version 1.13
-                    // cout << "\n         ******  "
-                    //     << iteration << T_GaMa_adjustment_iteration
-                    //     << "  ******\n\n"
-                    //     << T_GaMa_Approximate_coordinates_replaced << "\n"
-                    //     << underline(T_GaMa_Approximate_coordinates_replaced,
-                    //                  '*') << "\n\n\n";
-
-                    IS->refine_approx();
-                    // GeneralParameters(IS, cout);
-                  }
-                // FixedPoints     (IS, cout);
-                // AdjustedUnknowns(IS, cout);
+                IS->increment_linearization_iterations();
+                IS->refine_approx();
               }
-            while (TestLinearization(IS) && iteration < 5);
 
-            // from version 1.13 running results are not printed
-            if (iteration > 1)
+            if (IS->linearization_iterations() > 0)
               {
                 cout << T_GaMa_Approximate_coordinates_replaced << "\n"
                      << underline(T_GaMa_Approximate_coordinates_replaced,'*')
                      << "\n\n"
                      << T_GaMa_Number_of_linearization_iterations
-                     << iteration << "\n\n";
-
-                if (!TestLinearization(IS, cout)) cout << "\n";
+                     << IS->linearization_iterations() << "\n\n";
               }
 
+            if (!TestLinearization(IS, cout)) cout << "\n";
+
             NetworkDescription   (IS->description, cout);
             GeneralParameters    (IS, cout);
             FixedPoints          (IS, cout);
@@ -592,6 +601,21 @@ int main(int argc, char **argv)
                 xml.write(file);
               }
           }
+
+        if (network_can_be_adjusted && argv_updated_xml)
+          {
+            std::string xml = IS->updated_xml();
+
+            if (!strcmp(argv_updated_xml, "-"))
+              {
+                std::cout << xml;
+              }
+            else
+              {
+                ofstream file(argv_updated_xml);
+                file << xml;
+              }
+          }
       }
 
     delete IS;
