@@ -1,7 +1,7 @@
 /* GNU Gama -- adjustment of geodetic networks
     Copyright (C) 1999, 2006, 2010  Ales Cepek <cepek@fsv.cvut.cz>
                   2011  Vaclav Petras <wenzeslaus@gmail.com>
-                  2012, 2013, 2014  Ales Cepek <cepek@gnu.org>
+                  2012, 2013, 2014, 2015  Ales Cepek <cepek@gnu.org>
 
    This file is part of the GNU Gama C++ library.
 
@@ -27,6 +27,7 @@
  */
 
 #include <fstream>
+#include <sstream>
 #include <iomanip>
 #include <cctype>
 #include <memory>
@@ -215,10 +216,11 @@ LocalNetwork::LocalNetwork()
     tst_rov_opr_(false), tst_vyrovnani_(false), min_n_(0), min_x_(0),
     gons_(true)
 {
-  least_squares = 0;
-  Asp = 0;
+  least_squares = nullptr;
+  Asp = nullptr;
 
-  xml_covband_   = -1;
+  set_adj_covband();
+  set_max_linearization_iterations();
   // epoch_         = 0.0;
   // has_epoch_     = false;
   // latitude_      = M_PI/4.0;
@@ -281,23 +283,36 @@ void LocalNetwork::set_algorithm(std::string alg)
   algorithm_ = alg;
   has_algorithm_ = true;
 
-  if (least_squares != 0) delete least_squares;
+  if (least_squares != nullptr) delete least_squares;
   least_squares = adjb;
 
   update(Points);
 }
 
 
-int LocalNetwork::xml_covband() const
+int LocalNetwork::adj_covband() const
 {
-  return xml_covband_;
+  return adj_covband_;
 }
 
 
-void LocalNetwork::set_xml_covband(int val)
+void LocalNetwork::set_adj_covband(int val)
 {
   if (val < -1) val = -1;
-  xml_covband_ = val;
+  adj_covband_ = val;
+}
+
+
+int LocalNetwork::max_linearization_iterations() const
+{
+  return max_linearization_iterations_;
+}
+
+
+void LocalNetwork::set_max_linearization_iterations(int value)
+{
+  if (value < 0) value = 0;
+  max_linearization_iterations_ = value;
 }
 
 
@@ -392,7 +407,7 @@ void LocalNetwork::revision_points()
             }
           else
             {
-              b.unused_xy();
+              b.set_unused_xy();
               removed( (*bod).first, rm_missing_xy );
               ok = false;
             }
@@ -406,7 +421,7 @@ void LocalNetwork::revision_points()
             }
           else
             {
-              b.unused_z();
+              b.set_unused_z();
               removed( (*bod).first, rm_missing_z );
               ok = false;
             }
@@ -631,7 +646,7 @@ void LocalNetwork::project_equations()
       // design matrix
 
       input.set_mat(Asp);
-      Asp = 0;
+      Asp = nullptr;
 
       // ---  cofactors  --------------------------------------------------
 
@@ -682,7 +697,7 @@ void LocalNetwork::project_equations()
   //--opr.write((char*)(A.begin()), sizeof(Double)*m*n);
 
   delete[] min_x_;
-  min_x_ = 0;
+  min_x_ = nullptr;
   min_n_ = 0;
 
   for (PointData::iterator i=PD.begin(); i!=PD.end(); ++i)
@@ -709,8 +724,8 @@ void LocalNetwork::project_equations()
               min_x_[n++] = p.index_z();
             }
         }
-      least_squares->min_x(min_n_, min_x_);
     }
+  least_squares->min_x(min_n_, min_x_);
 
   tst_rov_opr_ = true;
   update(Adjustment);
@@ -732,7 +747,7 @@ bool LocalNetwork::singular_coords(const Mat& A)
       if(p.index_x() == 0 || p.index_y() == 0)
       {
           result = true;
-          p.unused_xy();
+          p.set_unused_xy();
           removed( (*i).first, rm_singular_xy );
           continue;
       }
@@ -750,12 +765,6 @@ bool LocalNetwork::singular_coords(const Mat& A)
           bb += b*b;
         }
 
-      // old scale dependent test:
-      //
-      // D = aa*bb - ab*ab;
-      //
-      // if ((aa == 0) || (fabs(D) <= aa*1e-6))
-
       if (bb > aa) std::swap(aa, bb);
       if (aa == 0)
         D = 0;
@@ -765,7 +774,7 @@ bool LocalNetwork::singular_coords(const Mat& A)
       if (D < 1e-12)
         {
           result = true;
-          p.unused_xy();
+          p.set_unused_xy();
           removed( (*i).first, rm_singular_xy );
         }
 
@@ -920,11 +929,6 @@ Double LocalNetwork::test_abs_term(Index indm)
 {
   Observation* m = RSM[indm-1];
 
-  // 2005-12-28 added test for coordinates and vectors
-  //
-  // if (dynamic_cast<const Coordinates*>(m->ptr_cluster())) return 0;
-  // if (dynamic_cast<const Vectors*>(m->ptr_cluster())) return 0;
-
   const LocalPoint& stan = PD[m->from()];
   const LocalPoint& cil  = PD[m->to()];   // ignoring second angle target here
 
@@ -970,12 +974,12 @@ int LocalNetwork::null_space()
             LocalPoint& p = PD[id];
             if (type == 'X' || type == 'Y' || type == 'R' )
               {
-                p.unused_xy();
+                p.set_unused_xy();
                 removed(id, rm_singular_xy );
               }
             else if (type == 'Z' )
               {
-                p.unused_z();
+                p.set_unused_z();
                 removed(id, rm_missing_z );
               }
 
@@ -1085,20 +1089,6 @@ void LocalNetwork::forwardSubstitution(const CovMat& chol, Vec& v)
     }
 }
 
-// void LocalNetwork::backwardSubstitution(const Cov& chol, Vec& v)
-// {
-//   using namespace std;
-//   const Index N = chol.rows();
-//   const Index b = chol.bandWidth();
-//
-//   for (Index i=N; i>0; i--)
-//     {
-//       Index m = min(N, i+b);
-//       for (Index j=i+1; j<=m; j++) v(i) -= chol(i,j)*v(j);
-//
-//       v(i) /= chol(i,i);
-//     }
-// }
 
 void LocalNetwork::prepareProjectEquations()
 {
@@ -1142,19 +1132,19 @@ void LocalNetwork::vyrovnani_()
   using namespace GNU_gama::local;
   if (tst_vyrovnani_) return;
 
-  project_equations();
-  if (sum_unknowns()     == 0)
-    throw GNU_gama::local::Exception(T_GaMa_No_unknowns_defined);
-  if (sum_observations() == 0)
-    throw GNU_gama::local::Exception(T_GaMa_No_observations_available);
-  if (sum_points()      == 0)
-    throw GNU_gama::local::Exception(T_GaMa_No_points_available);
+  do {
 
-  /* least_squares->solve(); ... removed in 1.9.01a */
+    project_equations();
+    if (sum_unknowns()     == 0)
+      throw GNU_gama::local::Exception(T_GaMa_No_unknowns_defined);
+    if (sum_observations() == 0)
+      throw GNU_gama::local::Exception(T_GaMa_No_observations_available);
+    if (sum_points()      == 0)
+      throw GNU_gama::local::Exception(T_GaMa_No_points_available);
 
-  tst_vyrovnani_ = true;
+    tst_vyrovnani_ = true;
 
-  { /* ----------------------------------------------------------------- */
+    /* ----------------------------------------------------------------- */
     // check for huge covariances / indefinite coordinates
 
     for (PointData::iterator i=PD.begin(); i!=PD.end(); ++i)
@@ -1165,40 +1155,35 @@ void LocalNetwork::vyrovnani_()
         if (!P.free_xy() && !P.free_z()) continue;
 
         Double tx=0, ty=0, tz=0;
-        if (int ix = P.index_x()) tx = m_0_apr_ * sqrt( least_squares->q_xx(ix,ix) );
-        if (int iy = P.index_y()) ty = m_0_apr_ * sqrt( least_squares->q_xx(iy,iy) );
-        if (int iz = P.index_z()) tz = m_0_apr_ * sqrt( least_squares->q_xx(iz,iz) );
+        if (int ix = P.index_x()) tx =m_0_apr_*sqrt(least_squares->q_xx(ix,ix));
+        if (int iy = P.index_y()) ty =m_0_apr_*sqrt(least_squares->q_xx(iy,iy));
+        if (int iz = P.index_z()) tz=m_0_apr_*sqrt(least_squares->q_xx(iz,iz));
 
         bool bxy = (tx > 1e4) || (ty > 1e4);
         bool bz  = (tz > 1e4);
 
         if (bxy && bz)
           {
-            P.unused_xy();
-            P.unused_z();
+            P.set_unused_xy();
+            P.set_unused_z();
             removed(id, rm_huge_cov_xyz);
-            tst_vyrovnani_ = false;
+            update(Points);
           }
         else if (bxy)
           {
-            P.unused_xy();
+            P.set_unused_xy();
             removed(id, rm_huge_cov_xy);
-            tst_vyrovnani_ = false;
+            update(Points);
           }
         else if (bz)
           {
-            P.unused_z();
+            P.set_unused_z();
             removed(id, rm_huge_cov_z);
-            tst_vyrovnani_ = false;
+            update(Points);
           }
       }
 
-    if (!tst_vyrovnani_)
-      {
-        vyrovnani_();
-        return;
-      }
-  }
+  } while (!tst_vyrovnani_);
 
   if (AdjBaseFull* full = dynamic_cast<AdjBaseFull*>(least_squares))
   { /* ----------------------------------------------------------------- */
@@ -1265,7 +1250,7 @@ void LocalNetwork::vyrovnani_()
           const Clust_r& cluster = *(*cit);
           // ??? if (cluster.covariance_matrix.bandWidth())
           // ???   {
-          // ???     // vypocet pro korelovana mereni zatim chybi !!!
+          // ???     // calculation for correlated observations missing !!!
           // ???   }
           // ??? else
             {
@@ -1299,3 +1284,264 @@ void LocalNetwork::vyrovnani_()
   }
 
 }
+
+
+std::string LocalNetwork::updated_xml()
+{
+  if (!is_adjusted()) std::string();
+
+  std::string xml =
+    "<?xml version=\"1.0\" ?>\n"
+    "<gama-local xmlns=\"http://www.gnu.org/software/gama/gama-local\">\n"
+    "<network axes-xy=";
+
+  switch(PD.local_coordinate_system)
+    {
+    case LocalCoordinateSystem::EN: xml += "\"en\""; break;
+    case LocalCoordinateSystem::NW: xml += "\"nw\""; break;
+    case LocalCoordinateSystem::SE: xml += "\"se\""; break;
+    case LocalCoordinateSystem::WS: xml += "\"ws\""; break;
+    case LocalCoordinateSystem::NE: xml += "\"ne\""; break;
+    case LocalCoordinateSystem::SW: xml += "\"sw\""; break;
+    case LocalCoordinateSystem::ES: xml += "\"es\""; break;
+    case LocalCoordinateSystem::WN: xml += "\"wn\""; break;
+    default:
+      xml +=  "\"ne\""; break;
+    }
+
+  xml += " angles=";
+  xml += PD.left_handed_angles() ? "\"left-handed\"" : "\"right-handed\"";
+  if (has_epoch()) xml += " epoch=\"" + std::to_string(epoch()) + "\"";
+  xml += ">\n";
+
+
+  if (!description.empty())
+    xml += "\n<description>" + description + "</description>\n";
+
+  xml += "\n<parameters\n";
+  xml += "  sigma-apr=\"" + std::to_string(apriori_m_0()) + "\"\n";
+  xml += "  conf-pr=\""   + std::to_string(conf_pr())     + "\"\n";
+  xml += "  tol-abs=\""   + std::to_string(tol_abs())     + "\"\n";
+  xml += "  sigma-act=\"";
+  xml +=         m_0_apriori() ? "apriori\"\n" : "aposteriori\"\n";
+  xml += "  update-constrained-coordinates=\"";
+  xml +=         update_constrained_coordinates() ? "yes\"\n" : "no\"\n";
+  xml += "  angles=\"" + std::string(gons() ? "400" : "360") + "\"\n";
+  if (has_algorithm()) xml += "  algorithm=\"" + algorithm() + "\"\n";
+  if (has_latitude())
+    xml += "  latitude=\"" + std::to_string(latitude()) + "\"\n";
+  if (has_ellipsoid()) xml += "  ellipsoid=\"" + ellipsoid() + "\"\n";
+  xml += "  cov-band=\"" + std::to_string(adj_covband()) + "\"\n";
+  // iterations ... not implemented in XML input
+  // language .....
+  // encoding .....
+  xml += "/>\n";
+
+
+  xml += "\n<points-observations";
+  // implicit parameters
+  xml += ">\n\n";
+
+
+  for (auto p=PD.begin(); p!=PD.end(); ++p)
+    {
+      PointID    id    = p->first;
+      LocalPoint point = p->second;
+      if (!point.active()) continue;
+
+      xml += "<point id=\"" + id.str() + "\"";
+
+      if (point.test_xy()) {
+        xml += " x=\"" + std::to_string(point.x()) + "\"";
+        xml += " y=\"" + std::to_string(point.y()) + "\"";
+      }
+
+      if (point.test_z()) {
+          xml += " z=\"" + std::to_string(point.z()) + "\"";
+      }
+
+      std::string fix {}, adj {};
+      if      (point.fixed_xy())       fix += "xy";
+      else if (point.constrained_xy()) adj += "XY";
+      else if (point.free_xy())        adj += "xy";
+
+      if      (point.fixed_z())        fix += "z";
+      else if (point.constrained_z())  adj += "Z";
+      else if (point.free_z())         adj += "z";
+
+      if (!fix.empty()) xml += " fix=\"" + fix + "\"";
+      if (!adj.empty()) xml += " adj=\"" + adj + "\"";
+
+      xml += " />\n";
+    }
+
+
+  DisplayObservationVisitor info(this);
+
+  for (auto c=OD.clusters.begin(); c!=OD.clusters.end(); ++c) {
+    if (auto cluster = dynamic_cast<StandPoint*>(*c))
+      {
+        xml += "\n<obs from=\"" + cluster->station.str() + "\">\n";
+
+        for (auto p  = cluster->observation_list.begin();
+                  p != cluster->observation_list.end(); ++p) {
+          Observation* obs = *p;
+          obs->accept(&info);
+
+          xml += "<" +  info.xml_name;
+          if (!info.str_to.empty()) {
+            xml += " to=\"" + info.str_to + "\"";
+            double fdh = obs->from_dh();
+            double tdh = obs->to_dh();
+            if (fdh) xml += " from_dh=\"" + std::to_string(fdh) + "\"";
+            if (tdh) xml +=   " to_dh=\"" + std::to_string(tdh) + "\"";
+          }
+          else if (!info.str_bs.empty()) {
+            xml += " bs=\"" + info.str_bs + "\"";
+            xml += " fs=\"" + info.str_fs + "\"";
+
+            Angle* a = static_cast<Angle*>(obs);
+            double rdh = a->from_dh();
+            double bdh = a->bs_dh();
+            double fdh = a->fs_dh();
+            if (rdh) xml += " from_dh=\"" + std::to_string(rdh) + "\"";
+            if (bdh) xml +=   " bs_dh=\"" + std::to_string(bdh) + "\"";
+            if (fdh) xml +=   " fs_dh=\"" + std::to_string(fdh) + "\"";
+          }
+          xml += " val=\"" + info.str_val + "\"";
+          xml += " stdev=\"" + info.str_stdev + "\"";
+          xml += " />\n";
+        }
+
+        updated_xml_covmat(xml, cluster->covariance_matrix, false);
+         xml += "</obs>\n";
+      }
+    else if (auto cluster = dynamic_cast<HeightDifferences*>(*c))
+      {
+        xml += "\n<height-differences>\n";
+        for (auto p  = cluster->observation_list.begin();
+                  p != cluster->observation_list.end(); ++p) {
+          Observation* obs = *p;
+          obs->accept(&info);
+
+         xml += "<" +  info.xml_name;
+         xml += " from=\"" + info.str_from + "\"";
+         xml += " to=\"" + info.str_to + "\"";
+         xml += " val=\"" + info.str_val + "\"";
+
+         double dist = 0;
+         if (H_Diff* p = dynamic_cast<H_Diff*>(obs)) dist = p->dist();
+         if (dist > 0)
+           xml += " dist=\"" +std::to_string(dist) + "\"";
+         else
+           xml += " stdev=\"" + info.str_stdev + "\"";
+         xml += "/>\n";
+        }
+
+        updated_xml_covmat(xml, cluster->covariance_matrix, false);
+        xml += "</height-differences>\n";
+      }
+    else if (auto cluster = dynamic_cast<Coordinates*>(*c))
+      {
+        xml += "\n<coordinates>\n";
+        for (auto p  = cluster->observation_list.begin();
+                  p != cluster->observation_list.end(); ++p) {
+          Observation* obs = *p;
+          obs->accept(&info);
+          std::string from = info.str_from;
+          xml += "<point id=\"" + from + "\"";
+          if (info.xml_name == "x")
+            {
+              xml += " x=\"" + info.str_val + "\"";
+              ++p;
+              (*p)->accept(&info);
+              xml += " y=\"" + info.str_val + "\"";
+              auto q = p;
+              ++q;
+              if (q != cluster->observation_list.end())
+                {
+                  DisplayObservationVisitor tmp(this);
+                  (*q)->accept(&tmp);
+                  if (tmp.str_from == from && tmp.xml_name == "z")
+                    {
+                      ++p;
+                      (*p)->accept(&info);
+                    }
+                }
+            }
+          if (info.xml_name == "z")
+            {
+              xml += " z=\"" + info.str_val + "\"";
+            }
+          xml += " />\n";
+        }
+
+        updated_xml_covmat(xml, cluster->covariance_matrix, true);
+        xml += "</coordinates>\n";
+      }
+    else if (auto cluster = dynamic_cast<Vectors*>(*c))
+      {
+        xml += "\n<vectors>\n";
+
+        for (auto p  = cluster->observation_list.begin();
+                  p != cluster->observation_list.end(); ++p) {
+          Observation* obs = *p;
+          obs->accept(&info);
+
+          xml += "<vec from=\"" +info.str_from+ "\" to=\"" +info.str_to+ "\"";
+          obs->accept(&info);
+          xml += " dx=\"" + info.str_val + "\"";
+          (*++p)->accept(&info);
+          xml += " dy=\"" + info.str_val + "\"";
+          (*++p)->accept(&info);
+          xml += " dz=\"" + info.str_val + "\"";
+
+          // double fdh = (*p)->from_dh(); ... unrealistic
+          // double tdh = (*p)->to_dh();
+          // if (fdh) xml += " from_dh=\"" + std::to_string(fdh) + "\"";
+          // if (tdh) xml +=   " to_dh=\"" + std::to_string(tdh) + "\"";
+          xml += " />\n";
+        }
+
+        updated_xml_covmat(xml, cluster->covariance_matrix, true);
+        xml += "\n</vectors>\n";
+      }
+    else
+      {
+        xml += "\n### Undefined cluster\n";
+      }
+  }
+
+
+  xml +=
+    "\n</points-observations>\n"
+    "\n</network>\n"
+    "</gama-local>\n";
+
+  return xml;
+}
+
+void LocalNetwork::updated_xml_covmat(std::string& xml, const CovMat& C,
+                                      bool always)
+{
+  Index  dim  = C.dim();
+  Index  band = C.bandWidth();
+  if (!always && band == 0) return;
+
+  xml += "\n<cov-mat dim=\"" + std::to_string(dim) + "\"";
+  xml += " band=\"" + std::to_string(band) + "\">\n";
+  for (Index i=1; i<=dim; i++)
+    {
+      std::ostringstream out;
+      out.setf(ios_base::scientific, ios_base::floatfield);
+      out.precision(16);
+      out << "\n";
+      for (Index n=1, j=i; j<=i+band && j <=dim; j++, n++)
+        {
+          out << setw(24) << C(i,j) << ((n%3 == 0 && j != dim) ? "\n" : " ");
+        }
+      out << "\n";
+      xml += out.str();
+    }
+  xml += "\n</cov-mat>\n";
+}
