@@ -277,8 +277,8 @@ namespace GNU_gama { namespace local
     std::string algorithm() const;
     bool        has_algorithm() const;
     void        set_algorithm(std::string alg = std::string());
-    int         xml_covband() const;
-    void        set_xml_covband(int value=-1);
+    int         adj_covband() const;
+    void        set_adj_covband(int value=1);
     double      epoch() const;
     bool        has_epoch() const;
     void        set_epoch(double=0.0);
@@ -291,12 +291,30 @@ namespace GNU_gama { namespace local
     bool        correction_to_ellipsoid() const;
     void        clear_nullable_data();
 
+    // ... linearization iterations ........................................
+
+    void set_max_linearization_iterations(int value=5);
+    int  max_linearization_iterations() const;
+    int  clear_linearization_iterations() { return (iterations_ = 0); }
+    int  linearization_iterations() const { return iterations_; }
+    int  increment_linearization_iterations() { return ++iterations_; }
+    bool next_linearization_iterations() const { return iterations_ < max_linearization_iterations_;}
+
+    // ...  input data with free coordinates updated after adjustment
+    //      and rejected observations removed ..............................
+
+    std::string updated_xml();
+
     // #####################################################################
 
 
   private:
 
-    int         xml_covband_;         // output XML xyz cov bandWidth
+    void updated_xml_covmat(std::string& xml, const CovMat& C, bool always);
+
+    int         adj_covband_;         // output XML xyz cov bandWidth
+    int         max_linearization_iterations_;
+    int         iterations_ {};
     std::string algorithm_;           // algorithm name or empty string
     bool        has_algorithm_;
     double      epoch_;
