@@ -1,5 +1,6 @@
 /* GNU Gama -- adjustment of geodetic networks
    Copyright (C) 2012  Ales Cepek <cepek@gnu.org>
+                 2014  Maxime Le Moual <maxime.le-moual@ensg.eu>
 
    This file is part of the GNU Gama C++ library.
 
@@ -21,6 +22,7 @@
  * \brief #GNU_gama::local::GamaLocalSVG class header file
  *
  * \author Ales Cepek
+ * \author Maxime Le Moual
  */
 
 #ifndef GAMA_LOCAL_SVG__Gama_Local_Svg__gama_local_svg__h
@@ -52,7 +54,7 @@ namespace GNU_gama { namespace local {
       void setDrawPointSymbols(bool p) {tst_draw_point_symbols = p; }
 
       /** Draw point IDs. */
-      bool drawPoinsIDs() const { return tst_draw_point_ids; }
+      bool drawPointIDs() const { return tst_draw_point_ids; }
       /** Set drawing of point IDs. */
       void setDrawPointIDs(bool p) {tst_draw_point_ids = p; }
 
@@ -62,7 +64,7 @@ namespace GNU_gama { namespace local {
       void setDrawEllipses(bool p) {tst_draw_ellipses = p; }
 
       /** Draw observations. */
-      bool drawPoinsObservations() const { return tst_draw_observations; }
+      bool drawObservations() const { return tst_draw_observations; }
       /** Set drawing of point symbols. */
       void setDrawObservations(bool p) {tst_draw_observations = p; }
 
@@ -72,49 +74,74 @@ namespace GNU_gama { namespace local {
       void setDrawAxes(bool p) {tst_draw_axes = p; }
 
       /** Font size. */
-      int fontSize() const { return fontsize; }
+      double fontSize() const { return fontsize; }
       /** Set font size */
-      void setFontSize(int p) { fontsize = p;}
+      void setFontSize(double p) { fontsize = p; tst_implicit_size = false; }
 
       /** Symbol size. */
-      int symbolSize() const { return symbolsize; }
+      double symbolSize() const { return symbolsize; }
       /** Set symbol size */
-      void setSymbolSize(int p) { symbolsize = p;}
+      void setSymbolSize(double p) { symbolsize = p; tst_implicit_size = false; }
 
       /** SVG stroke width. */
-      int strokeWidth() const { return strokewidth; }
+      double strokeWidth() const { return strokewidth; }
       /** Set SVG stroke size */
-      void setStrokeWidth(int p) { strokewidth = p;}
+      void setStrokeWidth(double p) { strokewidth = p; tst_implicit_size = false; }
 
       /** Fixed points' symbol. */
       std::string fixedSymbol() const { return fixedsymbol; }
       /** Set symbol for fixed points */
-      void setFixedSymbol(std::string p) { fixedsymbol = p;}
+      void setFixedSymbol(std::string p) { fixedsymbol = p; }
 
       /** Constrained points' symbol. */
       std::string constrainedSymbol() const { return constrainedsymbol; }
       /** Set symbol for fixed points */
-      void setConstrainedSymbol(std::string p) { constrainedsymbol = p;}
+      void setConstrainedSymbol(std::string p) { constrainedsymbol = p; }
 
       /** Free points' symbol. */
       std::string freeSymbol() const { return freesymbol; }
       /** Set symbol for fixed points */
-      void setFreeSymbol(std::string p) { freesymbol = p;}
+      void setFreeSymbol(std::string p) { freesymbol = p; }
 
       /** Fixed points' fill. */
       std::string fixedFill() const { return fixedfill; }
       /** Set fill for fixed points */
-      void setFixedFill(std::string p) { fixedfill = p;}
+      void setFixedFill(std::string p) { fixedfill = p; }
 
       /** Constrained points' fill. */
       std::string constrainedFill() const { return constrainedfill; }
       /** Set fill for constrained points */
-      void setConstrainedFill(std::string p) { constrainedfill = p;}
+      void setConstrainedFill(std::string p) { constrainedfill = p; }
 
       /** Free points' fill. */
       std::string freeFill() const { return freefill; }
       /** Set symbol for fixed points */
-      void setFreeFill(std::string p) { freefill = p;}
+      void setFreeFill(std::string p) { freefill = p; }
+
+      /** Restores default program settings derived from given coordinates set. */
+      void restoreDefaults();
+
+      /** Minimal size of font size and stroke width. */
+      double minimalSize() const { return minimalsize; }
+
+      /** Minimal size of font size and stroke width. */
+      void setMinimalSize(double p = 0.0001) const { minimalsize = p > 0 ? p : 0.0001; }
+
+      /** Error ellipse scale. */
+      double ellipsesScale() const { return ellipsescale; }
+
+      /** Error ellipse scale. */
+      void setEllipsesScale(double p) const { ellipsescale = p; tst_implicit_size = false; }
+
+      /** Transformation from geodetic to SVG coordinates (x, y)
+       * \code{.cpp}
+       * xsvg = t11*x + t12*y + tx;
+       * ysvg = t21*x + t22*y + ty;
+       * \endcode
+       */
+      void SvgCoordinates(double& t11, double& t12,
+                          double& t21, double& t22,
+                          double& tx,  double& ty) const;
 
     private:
       LocalNetwork&          IS;
@@ -124,10 +151,12 @@ namespace GNU_gama { namespace local {
 
       mutable std::ostream*  svg;
 
+      mutable double T11, T12, T21, T22, Tx, Ty;
+
       // SVG coordinates bounding box and offset
-      mutable bool not_in_constructor;
-      mutable int  minx, maxx, miny, maxy, offset;
-      mutable double ab_median;
+      mutable bool tst_implicit_size;
+      mutable double  minx, maxx, miny, maxy, offset;
+      mutable double ab_median, minimalsize, ellipsescale;
       void svg_xy(const LocalPoint& point, double& x, double& y) const;
       void svg_draw_point  (const PointID& pid, const LocalPoint& point) const;
       void svg_point_shape (double x, double y,
@@ -137,12 +166,47 @@ namespace GNU_gama { namespace local {
       void svg_axes_xy     () const;
       void svg_points      () const;
       void svg_observations() const;
+      void svg_ellipse(const PointID& pid, double& a, double &b, double& alpha) const;
 
-      mutable int fontsize, symbolsize, strokewidth;
+      mutable double fontsize, symbolsize, strokewidth;
       mutable bool tst_draw_axes, tst_draw_point_symbols, tst_draw_point_ids,
         tst_draw_ellipses, tst_draw_observations;
       mutable std::string  fixedsymbol, fixedfill, constrainedsymbol,
         constrainedfill, freesymbol, freefill;
+
+
+      /* helper svg point class */
+
+      class Point {
+      public:
+        double x, y;
+
+        Point() {}
+        Point(double a, double b) : x(a), y(b) {}
+
+        friend std::ostream& operator<< (std::ostream& ostr, const Point& p) {
+            return ostr << p.x << "," << p.y << " ";
+        }
+        friend Point operator+(const GamaLocalSVG::Point& a, const Point& b) {
+          return Point(a.x+b.x, a.y+b.y);
+        }
+        friend Point operator-(const Point& a, const Point& b)
+        {
+          return Point(a.x-b.x, a.y-b.y);
+        }
+        friend Point operator*(const Point& a, double q)
+        {
+          return Point(a.x*q, a.y*q);
+        }
+      };
+
+      mutable Point TX, TY;
+
+      void sett(double tr11, double tr12, double tr21, double tr22) const
+      {
+        TX = Point(tr11, tr12);
+        TY = Point(tr21, tr22);
+      }
     };
 }}
 
