@@ -33,6 +33,7 @@
 #include <gnu_gama/intfloat.h>
 #include <gnu_gama/gon2deg.h>
 #include <gnu_gama/xsd.h>
+#include <matvec/unsigned.h>
 
 namespace {
   typedef std::pair<double, bool> DB_pair;
@@ -121,7 +122,7 @@ namespace GNU_gama { namespace local {
           case tag_angle     : return process_angle(atts);
           case tag_s_distance: return process_sdistance(atts);
           case tag_z_angle   : return process_zangle(atts);
-          case tag_dh        : return process_obs_dh(atts);
+            // xsd 0.91 case tag_dh        : return process_obs_dh(atts);
           case tag_azimuth   : return process_azimuth(atts);
           case tag_cov_mat   : return process_obs_cov(atts);
           default:
@@ -213,9 +214,9 @@ namespace GNU_gama { namespace local {
       case state_obs_zangle:
         state = state_obs;
         break;
-      case state_obs_dh:
-        state = state_obs;
-        break;
+      // xsd 0.91 case state_obs_dh:
+      // xsd 0.91   state = state_obs;
+      // xsd 0.91   break;
       case state_obs_azimuth:
         state = state_obs;
         break;
@@ -528,6 +529,13 @@ namespace GNU_gama { namespace local {
             return error(T_GKF_bad_network_configuration_unknown_parameter
                          + jmeno + " = " + hodnota);
           }
+        else if (jmeno == "angles")
+          {
+            if      (hodnota == "400") lnet.set_gons();
+            else if (hodnota == "360") lnet.set_degrees();
+            else error(T_GKF_bad_network_configuration_unknown_parameter
+                         + jmeno + " = " + hodnota);
+          }
         else if (jmeno == "algorithm")
           {
             lnet.set_algorithm(hodnota);
@@ -539,7 +547,7 @@ namespace GNU_gama { namespace local {
               return error(T_GKF_undefined_value_of_attribute
                            + jmeno + " = " + hodnota);
 
-            lnet.set_xml_covband(ival);
+            lnet.set_adj_covband(ival);
           }
         else if (jmeno == "latitude")
           {
@@ -738,13 +746,13 @@ namespace GNU_gama { namespace local {
     if      (st == "fixed"     ) SB[pp_id].set_fixed_xy();
     else if (st == "free"      ) SB[pp_id].set_free_xy();
     else if (st =="constrained") SB[pp_id].set_constrained_xy();
-    else if (st == "unused"    ) SB[pp_id].unused_xy();
+    else if (st == "unused"    ) SB[pp_id].set_unused_xy();
     else if (st != "")
       return error(T_GKF_undefined_point_type + st);
 
     if      (sh == "fixed" ) SB[pp_id].set_fixed_z();
     else if (sh == "free"  ) SB[pp_id].set_free_z();
-    else if (sh == "unused") SB[pp_id].unused_z();
+    else if (sh == "unused") SB[pp_id].set_unused_z();
     else if (sh != "")
       return error(T_GKF_undefined_height_type + sh);
 
@@ -1216,58 +1224,58 @@ namespace GNU_gama { namespace local {
 
 
 
-  int GKFparser::process_obs_dh(const char** atts)
-  {
-    /*  the function body was copied from process_dh(const char**)  */
-    /*  ##########################################################  */
-
-    string  nam, val, sfrom, sto,  sval, sstdev, sdist;
-    // ###### state = state_hdiff_dh;
-    state = state_obs_dh;
-
-    while (*atts)
-      {
-        nam = string(*atts++);
-        val = string(*atts++);
-
-        if      (nam == "from" ) sfrom  = val;
-        else if (nam == "to"   ) sto    = val;
-        else if (nam == "val"  ) sval   = val;
-        else if (nam == "stdev") sstdev = val;
-        else if (nam == "dist" ) sdist  = val;
-        else
-          return error(T_GKF_undefined_attribute_of_height_differences
-                       + nam + " = " + val);
-      }
-
-    if (sfrom == "") return error(T_GKF_missing_from_ID);
-    if (sto   == "") return error(T_GKF_missing_to_ID);
-    if (sval  == "") return error(T_GKF_missing_observed_value);
-
-    double dm;
-    if (!toDouble(sval, dm)) return error(T_GKF_bad_height_diff + sval);
-    double dd = 0;
-    if (sdist != "")
-      if (!toDouble(sdist, dd) || dd < 0)
-        return error(T_GKF_bad_distance + sdist);
-    double ds = lnet.apriori_m_0() * sqrt(dd);
-    if (sstdev != "")
-      if (!toDouble(sstdev, ds)) return error(T_GKF_illegal_standard_deviation);
-
-    try
-      {
-        H_Diff* hd = new H_Diff(sfrom, sto, dm, dd);
-        // ###### heightdifferences->observation_list.push_back( hd );
-        standpoint->observation_list.push_back( hd );
-        sigma.push_back(DB_pair(ds, false));
-      }
-    catch  (const /*GNU_gama::local::*/Exception &e)
-      {
-        error(e.what());
-      }
-
-    return 0;
-  }
+// xsd 0.91 int GKFparser::process_obs_dh(const char** atts)
+// xsd 0.91 {
+// xsd 0.91   /*  the function body was copied from process_dh(const char**)  */
+// xsd 0.91   /*  ##########################################################  */
+// xsd 0.91
+// xsd 0.91   string  nam, val, sfrom, sto,  sval, sstdev, sdist;
+// xsd 0.91   // ###### state = state_hdiff_dh;
+// xsd 0.91   state = state_obs_dh;
+// xsd 0.91
+// xsd 0.91   while (*atts)
+// xsd 0.91     {
+// xsd 0.91       nam = string(*atts++);
+// xsd 0.91       val = string(*atts++);
+// xsd 0.91
+// xsd 0.91       if      (nam == "from" ) sfrom  = val;
+// xsd 0.91       else if (nam == "to"   ) sto    = val;
+// xsd 0.91       else if (nam == "val"  ) sval   = val;
+// xsd 0.91       else if (nam == "stdev") sstdev = val;
+// xsd 0.91       else if (nam == "dist" ) sdist  = val;
+// xsd 0.91       else
+// xsd 0.91         return error(T_GKF_undefined_attribute_of_height_differences
+// xsd 0.91                      + nam + " = " + val);
+// xsd 0.91     }
+// xsd 0.91
+// xsd 0.91   if (sfrom == "") return error(T_GKF_missing_from_ID);
+// xsd 0.91   if (sto   == "") return error(T_GKF_missing_to_ID);
+// xsd 0.91   if (sval  == "") return error(T_GKF_missing_observed_value);
+// xsd 0.91
+// xsd 0.91   double dm;
+// xsd 0.91   if (!toDouble(sval, dm)) return error(T_GKF_bad_height_diff + sval);
+// xsd 0.91   double dd = 0;
+// xsd 0.91   if (sdist != "")
+// xsd 0.91     if (!toDouble(sdist, dd) || dd < 0)
+// xsd 0.91       return error(T_GKF_bad_distance + sdist);
+// xsd 0.91   double ds = lnet.apriori_m_0() * sqrt(dd);
+// xsd 0.91   if (sstdev != "")
+// xsd 0.91     if (!toDouble(sstdev, ds)) return error(T_GKF_illegal_standard_deviation);
+// xsd 0.91
+// xsd 0.91   try
+// xsd 0.91     {
+// xsd 0.91       H_Diff* hd = new H_Diff(sfrom, sto, dm, dd);
+// xsd 0.91       // ###### heightdifferences->observation_list.push_back( hd );
+// xsd 0.91       standpoint->observation_list.push_back( hd );
+// xsd 0.91       sigma.push_back(DB_pair(ds, false));
+// xsd 0.91     }
+// xsd 0.91   catch  (const /*GNU_gama::local::*/Exception &e)
+// xsd 0.91     {
+// xsd 0.91       error(e.what());
+// xsd 0.91     }
+// xsd 0.91
+// xsd 0.91   return 0;
+// xsd 0.91 }
 
 
   int GKFparser::process_cov(const char** atts)
@@ -1294,7 +1302,7 @@ namespace GNU_gama { namespace local {
       return error(T_GKF_cov_mat_bad_band_width + nam + " = " + val);
 
     if (idim  < 1) return error(T_GKF_cov_mat_bad_dim + nam + " = " + val);
-    if (iband < 0 || iband >= idim)
+    if (isNegative(iband) || iband >= idim)
       return error(T_GKF_cov_mat_bad_band_width + nam + " = " + val);
 
     return 0;
